//@author: vux
//@help: standard constant shader
//@tags: color
//@credits: 
#define TwoPi 6.283184
Texture2D texture2d <string uiname="Texture";>;

SamplerState g_samLinear : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

 
cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : VIEWPROJECTION;
	float4x4 tW: WORLD;        //the models world matrix
	float dim = 0.1;
	float widthScale = 1;
	float normOffset = 0;
	float3 translate = {0,0,0};
};


cbuffer cbPerObj : register( b1 )
{
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
	float4x4 tColor <string uiname="Color Transform";>;
};

struct VS_IN
{
	float4 PosO : POSITION;
	float4 TexCd : TEXCOORD0;
	float3 Norm : NORMAL;

};

struct vs2gs
{
    float4 PosWVP : POSITION;
    float4 TexCd : TEXCOORD0;
	float3 Norm : NORMAL;
};

struct gs2ps
{
    float4 PosWVP: SV_POSITION;
    float4 TexCd: TEXCOORD0;
};

vs2gs VS(VS_IN input)
{
    vs2gs Out = (vs2gs)0;
    Out.PosWVP  = mul(input.PosO,mul(tW,tVP))+float4(translate,0);;
    Out.Norm  = mul(float4(input.Norm,0),mul(tW,tVP)).xyz;
    Out.TexCd = mul(input.TexCd, tTex);
    return Out;
}


float4x4 Vrotate(float rotX, 
         float rotY, 
         float rotZ)
  {
   rotX *= TwoPi;
   rotY *= TwoPi;
   rotZ *= TwoPi;
   float sx = sin(rotX);
   float cx = cos(rotX);
   float sy = sin(rotY);
   float cy = cos(rotY);
   float sz = sin(rotZ);
   float cz = cos(rotZ);
 
   return float4x4( cz*cy+sz*sx*sy, sz*cx, cz*-sy+sz*sx*cy, 0,
                   -sz*cy+cz*sx*sy, cz*cx, sz*sy+cz*sx*cy , 0,
                    cx * sy       ,-sx   , cx * cy        , 0,
                    0             , 0    , 0              , 1);
  }


[maxvertexcount(4)]
void GS(point vs2gs input[1], inout TriangleStream<gs2ps> gsout)
{
	gs2ps o;
	
	float l = sqrt(input[0].Norm.x * input[0].Norm.x + input[0].Norm.y * input[0].Norm.y + input[0].Norm.z * input[0].Norm.z);
			
	float pitch = acos(input[0].Norm.x / sqrt(input[0].Norm.x * input[0].Norm.x + input[0].Norm.y * input[0].Norm.y)) * (input[0].Norm.y < 0 ? -1 : 1);
	float yaw = acos(input[0].Norm.z / input[0].Norm.r);

	o.PosWVP = input[0].PosWVP+float4(-dim*widthScale,-dim,normOffset,0);
	o.PosWVP = mul(o.PosWVP,Vrotate(pitch,yaw,l));
	o.TexCd = float4(0,0,0,0);
	
	gsout.Append(o);
	o.PosWVP = input[0].PosWVP+float4(dim*widthScale,-dim,normOffset,0);
	o.PosWVP = mul(o.PosWVP,Vrotate(pitch,yaw,l));
	o.TexCd = float4(0,1,0,0);
	
	gsout.Append(o);
	o.PosWVP = input[0].PosWVP+float4(-dim*widthScale,dim,normOffset,0);
	o.PosWVP = mul(o.PosWVP,Vrotate(pitch,yaw,l));
	o.TexCd = float4(1,0,0,0);
	
	gsout.Append(o);
	o.PosWVP = input[0].PosWVP+float4(dim*widthScale,dim,normOffset,0);
	o.PosWVP = mul(o.PosWVP,Vrotate(pitch,yaw,l));
	o.TexCd = float4(1,1,0,0);
	
	gsout.Append(o);
	
	gsout.RestartStrip();
}


float4 PS(gs2ps In): SV_Target
{
    float4 col = texture2d.Sample(g_samLinear,In.TexCd.xy) * cAmb;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}





technique10 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




